/*
 * Created on 08.07.2005
 *
 * TOODO JL: more testcases for the MiniBrowser
 */
package isac.gui.browser.minibrowser;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.net.URL;

import javax.swing.event.HyperlinkEvent;

import isac.gui.browser.minibrowser.MiniBrowser;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.UIActionOnLink;
import junit.framework.TestCase;

/**
 * MiniBrowser
 * 
 * This testcases should test the MiniBrowser independend from the
 * implementation. So only functional use cases are tested.
 * 
 * All tests have to be done after each other (synchronized). The MiniBrowser
 * throws an event if the browser contend is updated. This is signalized with
 * the page_update_event_ flag. Then the assertion can be done. Therefore this
 * class is implemented as event listender.
 * 
 * @author JL
 * @version $Revision$
 * @see isac.gui.browser.minibrowser.IMiniBrowser
 * @see isac.gui.browser.minibrowser.MiniBrowser
 *  
 */
public class TestMiniBrowser extends TestCase implements PropertyChangeListener {

	private boolean page_update_event_ = false;
	
	/*
	 * testNewMiniBrowserWithLocalPage
	 * 
	 * Tests the generation of a MiniBrowser and the representation of a local
	 * (isac internal) page (problem, excample, ...)
	 * 
	 * @author Johannes Loinig
	 */
	public synchronized void testNewMiniBrowserWithLocalPage() throws IOException {
		
		System.out
				.println("---BEGIN isac.functest.minibrowser.testNewMiniBrowserWithLocalPage");

		//IBrowserDialog dialog = (IBrowserDialog) new ExampleDialog(new Session("dummy"));
		MockProxy proxy_mock = new MockProxy(null);
		MiniBrowser mini_browser = new MiniBrowser(proxy_mock);
		
		UIActionOnLink action = new UIActionOnLink(
				EUIElement.UI_MINIBROWSER_LINK,
				EUIContext.UI_CONTEXT_CONTENT, null,
				"file:isac/src/java-tests/isac/gui/browser/minibrowser/test.html");

		this.page_update_event_ = false;
		mini_browser.addPropertyChangeListener(this);
		mini_browser
			.doUIAction(action);

		while(!this.page_update_event_);

		String result = mini_browser.getText();
		String result_first_line = result.substring(0, result.indexOf("\n"));
		String expected_first_line = "<html math=\"http://www.w3.org/1998/Math/MathML\" xlink=\"http://www.w3.org/1999/xlink\" xmlns=\"#DEFAULT\">";
		
		assertEquals(expected_first_line, result_first_line);
		assertEquals("isac/src/java-tests/isac/gui/browser/minibrowser/test.html",
				mini_browser.getPage().getFile());
		
		System.out
				.println("---END isac.functest.minibrowser.testNewMiniBrowserWithLocalPage");
	}

	/*
	 * testKlickOnLink
	 * 
	 * Tests if the right action is sent when a link is klicked.
	 * @author Johannes Loinig
	 */
	public synchronized void testKlickOnLink() throws IOException {
		
		String link1 = "file:isac/src/java-tests/isac/gui/browser/minibrowser/test.html";
		String link2 = "file:isac/src/java-tests/isac/gui/browser/minibrowser/test2.html";
		System.out
				.println("---BEGIN isac.functest.minibrowser.testKlickOnLink");

		//IBrowserDialog dialog = (IBrowserDialog) new ExampleDialog(new Session("dummy"));
		MockProxy proxy_mock = new MockProxy(null);
		MiniBrowser mini_browser = new MiniBrowser(proxy_mock);
		mini_browser.addPropertyChangeListener(this);

		this.page_update_event_ = false;
		mini_browser
				.setPage(link1);
		while(!this.page_update_event_);

		this.page_update_event_ = false;

		URL url = new URL(link2);
		HyperlinkEvent hyperlink_event = new HyperlinkEvent(mini_browser,
				HyperlinkEvent.EventType.ACTIVATED, url);
		
		mini_browser.fireHyperlinkUpdate(hyperlink_event);

		assertEquals("No Event fired.", proxy_mock.hyperlink_event_fired_, true);

		assertEquals(proxy_mock.link_, link2);

		System.out.println("---END isac.functest.minibrowser.testKlickOnLink");
	}

	/* 
	 * Catches the PropertyChangeEvent of the MiniBrowser.
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * 
	 * @param property_change_event the Event to be catched
	 */
	public void propertyChange(PropertyChangeEvent property_change_event) {
		System.out.println("huhuhu");
		System.out.println("string"+property_change_event.getPropertyName());
		if(property_change_event.getPropertyName().equals("page")) {
			this.page_update_event_ = true;
		}
	}

}