/**
 * @author Andreas Griesmayer
 * Created on Jul 5, 2003
 */

package isac.kestore;

import isac.util.KEStorePaths;

import java.io.IOException;

import org.apache.log4j.PropertyConfigurator;
import org.apache.xmlrpc.WebServer;

public class KEStore {

	/**
	 * starts the XML-RPC server for the KE-Store and assignes it to port 1030.
	 * All services are located in KEStoreServices
	 * 
	 * sets PropertyConfigurator for log4j
	 * 
	 * @param args
	 *            0: path to xmldata 1: path to log4j.properties
	 * 
	 * @see KEStoreServices
	 */

	public static void main(String[] args) {

		if (args.length != 1) {
			System.out
					.println("Only the path to the config file is allowed as one and only parameter");
			System.exit(1);
		}
		KEStorePaths.setConstants(args[0]);

		String path0, path1;
		path0 = KEStorePaths.XML_DATA_PATH;
		path1 = KEStorePaths.LOG4J_PROPERTIES;

		try {
			// Start the server.
			System.out.println("Server: starting XML-RPC server on port 1030");
			// fixed path, port: new WebServer(1030);
			WebServer server = new WebServer(KEStorePaths.KESTORE_SERVER_PORT
					.intValue());
			server.setParanoid(false);
			// Register our handler class.
			server.addHandler("KEStore", new KEStoreServices(path0));

		} catch (IOException e) {
			System.out.println("Server: could not start server: "
					+ e.getMessage());
		} finally {
			System.out
					.println("Server: registered HelloHandler class to 'hello'");
			System.out.println("Server: now accepting requests...");
		}
	}
}